from typing import Union, Optional

from csp_cbc_auth.cbc.session import CBCSession
from psc_auth_integration.entities.auth_user_session import AuthUserSession

from cbc_content_reporting.api_clients.base_api_class import ReportingBaseClass


class ContentReportingV1Api(ReportingBaseClass):
    """
    Reporting V1 API wrapper
    """

    def __init__(self, session: Union[AuthUserSession, CBCSession]):
        super().__init__(session)
        self.reporting_v1_url = f'{self.reporting_base_url}/v1/orgs'

    def get_manifest(self, org_key: str, device_id: int, manifest_signature_hash: Optional[str] = None,
                     time_range: Optional[str] = None):
        """
        Get manifest only gets the “manifest” resource
        :param org_key: str org key
        :param device_id: int device id
        :param manifest_signature_hash: str Manifest Hash on the sensor
        :param time_range: str time range. For example: -1d, -7d, -60d
        :return:
        """
        params = {}
        if manifest_signature_hash:
            params['manifest_signature_hash'] = manifest_signature_hash
        if time_range:
            params['range'] = time_range

        url = f'{self.reporting_v1_url}/{org_key}/devices/{device_id}/manifests'
        response = self.get(url=url, params=params)
        return response

    def get_latest_manifest(self, org_key: str, device_id: int):
        """
        Get the latest manifest for the device
        :param org_key: str org key
        :param device_id: int device id
        :return:
        """
        url = f'{self.reporting_v1_url}/{org_key}/devices/{device_id}/manifests/latest'
        response = self.get(url=url)
        return response

    def get_content(self, org_key: str, device_id: int, manifest_signature_hash: Optional[str] = None,
                    content_hash: str = None, notification_urn: str = None, time_range: Optional[str] = None):
        """
        Get content, takes into account “manifest” information
        and returns matching “content” resource
        :param org_key: str org key
        :param device_id: int device id
        :param manifest_signature_hash: str Manifest Hash on the sensor
        :param content_hash: str sha hash of content
        :param notification_urn: str notification urn
        :param time_range: str time range. For example: -1d, -7d, -60d
        :return:
        """
        params = {}
        if manifest_signature_hash:
            params['manifest_signature_hash'] = manifest_signature_hash
        if time_range:
            params['range'] = time_range
        if content_hash:
            params['content_hash'] = content_hash
        if notification_urn:
            params['notification_urn'] = notification_urn

        url = f'{self.reporting_v1_url}/{org_key}/devices/{device_id}/manifests/contents'
        response = self.get(url=url, params=params)
        return response

    def get_content_latest(self, org_key: str, device_id: int):
        """
        Get the latest content the device has
        :param org_key: str org key
        :param device_id: int device id
        :return:
        """
        url = f'{self.reporting_v1_url}/{org_key}/devices/{device_id}/manifests/contents/latest'
        response = self.get(url=url)
        return response
