import os
from datetime import datetime

import pytest
import tenacity
from cbc_content_pacing.enums.error_reason import ErrorReason
from cbc_content_pacing.utils import content_pacing_utils
from proto_content_pacing.manifest_pb2 import ManifestRequest, ManifestResponse, UpdateReport, SuccessfulDownloads, \
    ManifestError
from psc_protocol.util.enums import OsMajorVersion
from psc_server_support.api.api_factory import ApiFactory
from python_sensor_sim import PythonSensorSim
from python_sensor_sim.enums import SensorDeviceType

from cbc_content_reporting.api_clients.bandwidth_reporting_v1 import BandwidthReportingV1Api
from cbc_content_reporting.api_clients.content_reporting_v1 import ContentReportingV1Api
from cbc_content_reporting.tests.utils.auth import CDSSession


pytest_plugins = []
if os.environ.get('USE_OWNERSHIP_PLUGIN', False):
    pytest_plugins.append('cbc_content_reporting.tests.plugins.ownership_plugin')


@pytest.fixture()
def auth_session() -> CDSSession:
    """
    Create new CSR auth session
    :return: CDSSession
    """
    return CDSSession()


@pytest.fixture()
def csr_auth_session() -> CDSSession:
    """
    Create new CSR auth session
    :return: CDSSession
    """
    return CDSSession(is_csr=True)


@pytest.fixture()
def org_device(auth_session: CDSSession) -> PythonSensorSim:
    """
    Creates an instance of python sensor sim
    - Sends status message to seed client version and platform
    - Sends manifest request to generate manifest on BE so it's later queryable from the reporting API
    :param auth_session: CDSSession
    :return Generator[PythonSensorSim, None, None]
    """
    device = PythonSensorSim(org_user=ApiFactory(auth_session.session), device_type=SensorDeviceType.WINDOWS)
    try:
        device.send_status_message(
            os_major=OsMajorVersion.WINDOWS_10,
            os_minor=0,
            client_version=f'3.0.{datetime.now().timestamp()}'
        )
        response = device.send_content_pacing_manifest(request_data=ManifestRequest())
        response.raise_for_status()

        yield device
    finally:
        device.uninstall_and_delete_device()


@pytest.fixture()
def manifest_response(org_device, content_reporting_v1_api, auth_session) -> ManifestResponse:
    """
    Send initial manifest requests to CPS to ensure manifest is generated for content + reporting API is updated
    for latest content
    :param org_device: PythonSensorSim
    :param content_reporting_v1_api: ContentReportingV1Api
    :param auth_session: CDSSession
    :return: ManifestResponse
    """
    response = org_device.send_content_pacing_manifest(request_data=ManifestRequest())
    response.raise_for_status()
    parsed_response = content_pacing_utils.parse_manifest_response(response)

    update_request = ManifestRequest(
        manifest=parsed_response.manifest.manifest_protobuf,
        update_report=UpdateReport(
            manifest_error=ManifestError(
                reason=ErrorReason.NO_MANIFEST_ERROR,
                error_message=''
            ),
            successful_downloads=[
                SuccessfulDownloads(
                    segment_type_id=seg.segment_type_id,
                    expected_signature_hash=seg.signature_hash,
                    elapsed_seconds=3,
                    bytes_received=seg.content_size_bytes
                ) for seg in parsed_response.manifest.manifest_segments
            ]
        )
    )

    @tenacity.retry(wait=tenacity.wait_fixed(2), stop=tenacity.stop_after_attempt(5), reraise=True)
    def wait_for_manifest_hash_update():
        empty_manifest_sha = 'c627c09c14e58e44bc51622dac392958ec88244e414b508020634f53cfcd1e69'

        update_response = org_device.send_content_pacing_manifest(request_data=update_request)
        update_response.raise_for_status()
        parsed_response = content_pacing_utils.parse_manifest_response(update_response)
        assert parsed_response.manifest.manifest_signature_hash.lower() != empty_manifest_sha

        report_response = content_reporting_v1_api.get_latest_manifest(org_key=auth_session.org_key,
                                                                       device_id=org_device.device_id)
        report_manifest_signature_hash = report_response.json().get('manifest_signature_hash')
        assert report_manifest_signature_hash.lower() != empty_manifest_sha
        assert parsed_response.manifest.manifest_signature_hash == report_manifest_signature_hash

        return parsed_response

    parsed_response = wait_for_manifest_hash_update()

    return parsed_response


@pytest.fixture()
def content_reporting_v1_api(csr_auth_session) -> ContentReportingV1Api:
    """
    Content Reporting V1 API wrapper
    :param csr_auth_session:
    :return:
    """
    return ContentReportingV1Api(csr_auth_session.session)

@pytest.fixture()
def bw_reporting_v1_api(csr_auth_session) -> BandwidthReportingV1Api:
    """
    Content Reporting V1 API wrapper
    :param csr_auth_session:
    :return:
    """
    return BandwidthReportingV1Api(csr_auth_session.session)
