import collections
import enum
import glob
import json
import logging
import os
import uuid
from pathlib import Path
from random import random, seed
from typing import Dict, List, Tuple

import pytest
import requests
from _pytest.main import Session


class SGUID(enum.Enum):
    psc_content_mgmt_app_cps = 'psc:content_mgmt:app:cps'
    psc_content_mgmt_app_rms = 'psc:content_mgmt:app:rms'
    psc_content_mgmt_app_reporting = 'psc:content_mgmt:app:reporting'


class TeamNames(enum.Enum):
    BOLT = 'Bolt'
    GRAY_MATTER = 'Gray Matter'
    ION = 'Ion'
    NGAV = 'NGAV'
    NIC_CAGE = 'Nic Cage'
    ORIGAMI = 'Origami'
    PINEAPPLE = 'Pineapple'
    RUBIK = 'Rubik'
    SUBZERO = 'Subzero'
    TAU = 'TAU'
    UNKNOWN = 'unknown'


SLACK_POST_URL = 'https://hooks.slack.com/services/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'


# Meta info on test ownership to sguid and vice versa
OWNERSHIP_MAP = {
    TeamNames.GRAY_MATTER: [SGUID.psc_content_mgmt_app_cps, SGUID.psc_content_mgmt_app_rms,
                            SGUID.psc_content_mgmt_app_reporting],
}

# Create a full list of Sguids
_list_of_sguids = list(OWNERSHIP_MAP.values())
_full_list = []
for _team_sguid in _list_of_sguids:
    if _team_sguid:
        _full_list.extend(_team_sguid)

NO_SGUID = 'NO_SGUID'

SLACK_CHANNEL = {
    TeamNames.GRAY_MATTER: '#team-graymatter-e2e-test-results',
}


class NotificationLevel(enum.Enum):
    ALL = 'ALL'  # Notify on success or failure
    FAILED = 'FAILED'  # Only notify on failure


NOTIFICATION_LEVEL = {
    TeamNames.GRAY_MATTER: NotificationLevel.ALL
}

#  Report failures only for the following errors
#  If not specified will report all failures
EXCEPTION_NOTIFICATION = {}


@pytest.hookimpl(tryfirst=True, hookwrapper=True)
def pytest_runtest_makereport(item, call):
    # Add the test to a new item on the session object if the test failed
    outcome = yield
    result = outcome.get_result()
    if not result.passed:
        results = getattr(item.session, 'results', {})
        excinfo = getattr(call, 'excinfo', None)
        if excinfo:
            result.exc_type = excinfo.typename

        results[item] = result
        item.session.results = results

    for mark in item.iter_markers(name='sguid'):
        sguids = getattr(item.session, 'sguids', set())
        sguids.add(mark.args[0].value)
        item.session.sguids = sguids


def pytest_sessionfinish(session: Session):
    # Collect all the test failures and associate them with
    # sguids in a [sguid] -> [test failure] relationship
    owner_fails: Dict[str, List[str]] = collections.defaultdict(lambda: [])
    failure_type = {}
    for item in getattr(session, 'items', []):
        test_result = getattr(item.session, 'results', {}).get(item)
        if test_result and test_result.failed:
            failure_type[item.name] = test_result.exc_type
            test_has_sguid = False
            for mark in item.iter_markers(name='sguid'):
                test_has_sguid = True
                owner_fails[mark.args[0].value].append(test_result.head_line)
            if not test_has_sguid:
                owner_fails[NO_SGUID].append(test_result.head_line)
    owner_fails['sguids'] = list(getattr(session, 'sguids', []))

    owner_fails['sguids'] = list(getattr(session, 'sguids', []))
    owner_fails['failure_types'] = failure_type

    # If there are test failures, write them to a temp json file
    #
    # This has to be done because sessonfinish only
    # knows about the node's tests with pytest-xdist
    if owner_fails:
        filename = Path(f'tmp-owner-{uuid.uuid4()}.json')
        if filename.exists():
            seed(random())
            filename = Path(f'tmp-owner-{uuid.uuid4()}.json')
        logging.info('Writing failure owners (%s) to : %s', owner_fails, filename)
        with filename.open('w') as file:
            json.dump(owner_fails, file)


def pytest_unconfigure(config):
    # Only run this code once the hub (and not the node) is completed
    if not hasattr(config, 'slaveinput'):
        sguid_failures, owner_totals, all_sguids = analyze_owner_sguid_totals()
        post_result_owners(owner_totals, all_sguids, sguid_failures)


def analyze_owner_sguid_totals() -> Tuple[dict, dict, set]:
    """
    Analyze test failures and determine ownership
    """
    all_data = []
    all_sguids = set()
    failure_types = {}
    sguid_failures: Dict[str, List[str]] = collections.defaultdict(lambda: [])

    owner_files = glob.glob('./tmp-owner-*')
    logging.info('Found failed files: %s', owner_files)
    for file in owner_files:
        with open(file, 'r') as open_file:
            all_data.append(json.load(open_file))
        os.remove(file)

    for data in all_data:
        for sguid in data.get('sguids'):
            all_sguids.add(sguid)
        failure_types.update(data.get('failure_types', {}))
        for sguid in data:
            sguid_failures[sguid].extend(data[sguid])

    # Create a new dict with data of [team owner] -> [test failure]
    # based on the team's ownership of sguids
    owner_totals: Dict[str, List[str]] = collections.defaultdict(lambda: [])
    for owner, sguids in OWNERSHIP_MAP.items():
        for sguid_to_check in sguids:
            if sguid_to_check == NO_SGUID:
                if NO_SGUID in sguid_failures.keys():
                    owner_totals[owner.value].extend(sguid_failures[NO_SGUID])
            elif sguid_to_check.value in sguid_failures.keys():
                test_names = sguid_failures[sguid_to_check.value]
                team_exceptions = EXCEPTION_NOTIFICATION.get(owner)
                if team_exceptions:
                    sguid_exceptions = team_exceptions.get(sguid_to_check)
                    if sguid_exceptions:
                        sguid_exceptions = [e.__name__ for e in sguid_exceptions]
                        filtered_failures = [k for k, v in failure_types.items() if v in sguid_exceptions]
                        base_test_name = [t.split('.')[-1] for t in test_names]
                        filtered_failures = [v for v in filtered_failures if v in base_test_name]
                        test_names = filtered_failures

                owner_totals[owner.value].extend(test_names)

    return sguid_failures, owner_totals, all_sguids


def post_result_owners(owner_totals: dict, all_sguids: set, sguid_failures: set = None):
    """
    Post results to all owner channels
    :param owner_totals: dict of owner : test failures
    """

    for owner, sguids in OWNERSHIP_MAP.items():
        failed_tests = owner_totals.get(owner.value)
        if failed_tests:
            post_result_owner(owner, failed_tests)
        elif NOTIFICATION_LEVEL.get(owner, NotificationLevel.FAILED) == NotificationLevel.ALL:
            if sguid_failures and any(sguid.value in sguid_failures.keys() for sguid in sguids):
                post_result_owner(owner, non_service_failures=True)
            elif any(sguid.value in all_sguids for sguid in sguids):
                post_result_owner(owner)


def post_result_owner(owner, failed_tests: list = None, non_service_failures=False):
    """
    Post test results to owner channel
    :param owner: owner of the tests
    :param failed_tests: Optional list of failed tests, if none it is assumed all tests passed
    """
    username = 'Happy Homer'
    icon_emoji = ':woohoo:'
    message_text = ':green_circle: All content reporting tests passed!'
    attachment_text = f'Environment being tested: {os.environ.get("TEST_ENV") or os.environ.get("CBC_ENV")}\n'

    if non_service_failures:
        username = 'Sad CBC Cloud'
        icon_emoji = ':sad_cloud_blue:'
        message_text = ':large_yellow_circle: Tests failed due to issues not related to reporting service.'

    if failed_tests:
        username = 'Eye of Sauron'
        icon_emoji = ':eye_of_sauron:'
        message_text = f':red_circle: There are {len(failed_tests)} failing content reporting tests.'
        attachment_text += 'Failed tests:\n'
        for failed_test in failed_tests:
            attachment_text += f'{failed_test}\n'

        if owner == TeamNames.UNKNOWN.value:
            message_text = f'There are {len(failed_tests)} failing ' \
                           f"tests that use SGUID(s) that aren't associated " \
                           f'with a team in ownership_plugin.py. Please update ' \
                           f'ownership_plugin.py with the SGUID(s) and the ' \
                           f'team(s) that are responsible for it.'

    build_url = os.environ.get('CI_PIPELINE_URL', 'https://localhost:8080')
    team_slack_data = {
        # use icon_url if you want to change to a hosted image for the user image
        # 'icon_url': 'https://gravatar.com/me',
        'icon_emoji': icon_emoji,
        'username': username,
        'channel': SLACK_CHANNEL[TeamNames(owner)],
        'text': message_text,
        'attachments': [
            {
                'text': attachment_text,
                'color': 'bad',
                'actions': [
                    {
                        'type': 'button',
                        'text': 'Investigate',
                        'url': f'{build_url}',
                        'style': 'primary'
                    }
                ]
            }
        ]
    }

    requests.post(SLACK_POST_URL, json=team_slack_data)
