import os

from csp_cbc_auth.interfaces.cbc import CBC
from psc_auth_integration.entities.auth_user_session import AuthUserSession

from cbc_content_reporting.tests.utils.env import is_csp


class CDSSession:
    """
    Object to manage auth session (CSP or Legacy) for calls to CDS
    """
    def __init__(self, session=None, is_csr: bool = False):
        """
        :param session: an already established auth session to use
        :param is_csr: bool True if a CSR auth session is desired, False if an org admin session is desired
        """
        self.is_csr = is_csr
        self.is_csp = is_csp()
        self._session = session
        self._org_id = None
        self._org_key = None
        self._user_id = None
        self._created_by = None

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self._session:
            self._session.close()

    @property
    def session(self):
        if not self._session:
            self.create_auth_session()
        return self._session

    @property
    def api_session(self):
        return self.session

    @property
    def headers(self):
        return self.api_session.headers

    @property
    def org_id(self):
        if not self._org_id:
            if self.is_csp:
                self._org_id = self.session.org_id
            else:
                self._org_id = self.session.get_user_org_id()
        return self._org_id

    @property
    def org_key(self):
        if not self._org_key:
            if self.is_csp:
                self._org_key = self.session.org_key
            else:
                self._org_key = self.session.get_user_org_key()
        return self._org_key

    @property
    def user_id(self):
        if not self._user_id:
            if self.is_csp:
                self._user_id = self.session.login_id
            else:
                self._user_id = self.session.get_user_id()
        return self._user_id

    @property
    def created_by(self):
        if not self._created_by:
            if self.is_csp:
                self._created_by = f'psc:cnn:{self.org_key}:{self.user_id}'
            else:
                self._created_by = f'psc:user:{self.org_key}:{self.user_id}'
        return self._created_by

    def create_auth_session(self):
        """
        Create auth session for use with CDS
        - If CSP env variables are dedicated create a CSP session otherwise default to legacy auth
        - If is_csr is passed to the object create a CSR session otherwise default to org admin
        """
        if self.is_csp:
            auth_client_id = os.environ.get('CSR_CSP_AUTH_CLIENT_ID') if self.is_csr \
                else os.environ.get('CSP_AUTH_CLIENT_ID')
            auth_client_secret = os.environ.get('CSR_CSP_AUTH_CLIENT_SECRET') if self.is_csr \
                else os.environ.get('CSP_AUTH_CLIENT_SECRET')

            session = CBC.auth_oauth_client_credentials(csp_auth_client_id=auth_client_id,
                                                        csp_auth_client_secret=auth_client_secret
                                                        ).cbc_session

            if self.is_csr:
                session.csr = self.is_csr
                session.gateway_url = session.url
        else:

            username = os.environ.get('CSR_USERNAME') if self.is_csr else os.environ.get('ORG_USERNAME')
            password = os.environ.get('CSR_PASSWORD') if self.is_csr else os.environ.get('ORG_PASSWORD')
            gateway_url = os.environ.get('CSR_URL') if self.is_csr else os.environ.get('ORG_URL')

            session = AuthUserSession(retry_on_service_error=True,
                                      username=username,
                                      password=password,
                                      gateway_url=gateway_url
                                      )
            session.org_key = session.get_user_org_key()
            session.org_id = session.get_user_org_id()
            session.url = gateway_url

        self._session = session
