import pytest
import tenacity
from cbc_content_pacing.entities.manifest import ManifestResponse
from python_sensor_sim import PythonSensorSim

from cbc_content_reporting.api_clients.content_reporting_v1 import ContentReportingV1Api
from cbc_content_reporting.tests.plugins.ownership_plugin import SGUID
from cbc_content_reporting.tests.utils.auth import CDSSession
from cbc_content_reporting.tests.utils.request import retry_request


@pytest.mark.sguid(SGUID.psc_content_mgmt_app_reporting)
class TestManifests:

    def test_get_manifest(self, auth_session: CDSSession, content_reporting_v1_api: ContentReportingV1Api,
                          org_device: PythonSensorSim, manifest_response: ManifestResponse):
        """
        Sanity check using simple call to get manifest
        :param auth_session: CDSSession
        :param content_reporting_v1_api: ContentReportingV1Api
        :param org_device: PythonSensorSim
        :param manifest_response: ManifestResponse
        :return:
        """
        manifest_signature_hash = manifest_response.manifest.manifest_signature_hash

        @tenacity.retry(wait=tenacity.wait_fixed(5), stop=tenacity.stop_after_attempt(12), reraise=True)
        def wait_for_sha():
            response = retry_request(
                request=content_reporting_v1_api.get_manifest,
                org_key=auth_session.org_key,
                device_id=org_device.device_id)
            results = response.json().get('results')
            self.verify_manifest_signature_hash(manifest_signature_hash, results)

        wait_for_sha()

    def test_get_latest_manifest(self, auth_session: CDSSession, content_reporting_v1_api: ContentReportingV1Api,
                                 org_device: PythonSensorSim, manifest_response: ManifestResponse):
        """
        Sanity check using simple call to get latest manifest
        :param auth_session: CDSSession
        :param content_reporting_v1_api: ContentReportingV1Api
        :param org_device: PythonSensorSim
        :param manifest_response: ManifestResponse
        :return:
        """
        manifest_signature_hash = manifest_response.manifest.manifest_signature_hash

        @tenacity.retry(wait=tenacity.wait_fixed(5), stop=tenacity.stop_after_attempt(12), reraise=True)
        def wait_for_sha():
            response = retry_request(
                request=content_reporting_v1_api.get_latest_manifest,
                org_key=auth_session.org_key,
                device_id=org_device.device_id)
            response_json = response.json()
            assert response_json.get('manifest_signature_hash') == manifest_signature_hash

        wait_for_sha()

    def test_get_content(self, auth_session: CDSSession, content_reporting_v1_api: ContentReportingV1Api,
                         org_device: PythonSensorSim, manifest_response: ManifestResponse):
        """
        Sanity check using simple call to get content
        :param auth_session: CDSSession
        :param content_reporting_v1_api: ContentReportingV1Api
        :param org_device: PythonSensorSim
        :return:
        """
        manifest_signature_hash = manifest_response.manifest.manifest_signature_hash
        response = retry_request(
            request=content_reporting_v1_api.get_content,
            org_key=auth_session.org_key,
            device_id=org_device.device_id,
            stop_after_attempt=6)
        results = response.json().get('results')
        self.verify_manifest_signature_hash(manifest_signature_hash, results)

    def test_get_content_latest(self, auth_session: CDSSession, content_reporting_v1_api: ContentReportingV1Api,
                                org_device: PythonSensorSim, manifest_response: ManifestResponse):
        """
        Sanity check using simple call to get latest content
        :param auth_session: CDSSession
        :param content_reporting_v1_api: ContentReportingV1Api
        :param org_device: PythonSensorSim
        :return:
        """
        manifest_signature_hash = manifest_response.manifest.manifest_signature_hash

        response = retry_request(
            request=content_reporting_v1_api.get_content_latest,
            org_key=auth_session.org_key,
            device_id=org_device.device_id)
        results = response.json().get('results')
        self.verify_manifest_signature_hash(manifest_signature_hash, results)

    def verify_manifest_signature_hash(self, manifest_signature_hash: str, results: list):
        """
        Verify signature hash is present in results
        :param manifest_signature_hash: hash to check
        :param results: list of results from reporting api
        """
        hashes = [r.get('manifest_signature_hash') for r in results]
        assert manifest_signature_hash in hashes
